package com.example.survey.service.impl;

import com.example.survey.dao.InvestigationRecordDao;
import com.example.survey.dao.RespondentDao;
import com.example.survey.dao.UserDao;
import com.example.survey.entity.InvestigationRecord;
import com.example.survey.entity.inner.OperationInformation;
import com.example.survey.exception.RecordException;
import com.example.survey.exception.RespondentException;
import com.example.survey.exception.UserException;
import com.example.survey.service.InvestigationRecordService;
import com.example.survey.util.WordUtil;
import com.example.survey.vo.InvestigationRecordVo;
import com.example.survey.vo.ReviewVo;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author Pope
 */
@Log4j2
@Service
public class InvestigationRecordServiceImpl implements InvestigationRecordService {


    @Autowired
    private InvestigationRecordDao investigationRecordDao;

    @Autowired
    private RespondentDao respondentDao;

    @Autowired
    private UserDao userDao;


    @Override
    public boolean addInvestigationRecord(InvestigationRecordVo vo) {
        //如果不存在对应电话号码的调查对象
        if (!respondentDao.existRespondent(vo.getIdNumber())) {
            log.error("调查对象不存在");
            throw new RespondentException("调查对象不存在");
        }
        //如果已经存在待审核或已审核记录则失败
        if (investigationRecordDao.existInvestigationRecord(vo.getIdNumber(), InvestigationRecord.UNDER_REVIEW, InvestigationRecord.REVIEWED)) {

            log.error("数据库已存在对应调查对象的待审核或已审核记录");
            throw new RecordException("数据库已存在对应调查对象的待审核或已审核记录");
        }
        //如果不存在对应电话号码的流调人员
        if(!userDao.existUser(vo.getUserPhone())){
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }

        InvestigationRecord record = new InvestigationRecord(vo);
        //生成版本信息
        record.setState(InvestigationRecord.UNDER_REVIEW);
        record.setVersion(vo.getUserPhone() + new Date().toString());
        List<OperationInformation> opList = new ArrayList<>();
        opList.add(OperationInformation.submitOp(vo.getUserPhone(), vo.getMsg()));
        record.setOperationInformationList(opList);


        //TODO  插入消息队列

        return investigationRecordDao.insertInvestigationRecord(record);
    }

    @Override
    public List<InvestigationRecord> listUnderReviewRecordLimit(String userPhone, int currentPage, int pageSize, String state, String idNumber, String version, String questionnaireNumber, Boolean diseased) {
        //TODO 从消息队列进行筛选

        return investigationRecordDao.listInvestigationRecordLimit(userPhone, pageSize * currentPage, pageSize, state, idNumber, version, questionnaireNumber, diseased);
    }


    @Override
    public long countUnderReviewRecord(String userPhone) {
        //TODO 从消息队列获取调查记录数量

        return investigationRecordDao.countInvestigationRecordByUserPhone(userPhone, InvestigationRecord.UNDER_REVIEW);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void changeInvestigationRecord(InvestigationRecordVo vo) {
        //创建新记录
        InvestigationRecord oldRecord = investigationRecordDao.selectInvestigationRecord(vo.getIdNumber(), InvestigationRecord.REVIEWED);
        if (oldRecord == null) {
            oldRecord = investigationRecordDao.selectInvestigationRecord(vo.getIdNumber(), InvestigationRecord.NOT_PASS);
        }

        if(oldRecord == null){
            log.error("调查记录不存在");
            throw new RecordException("调查记录不存在");
        }
        InvestigationRecord newRecord = new InvestigationRecord(vo);

        //生成版本信息
        newRecord.setState(InvestigationRecord.UNDER_REVIEW);
        newRecord.setVersion(vo.getUserPhone() + new Date().toString());
        //生成操作记录
        List<OperationInformation> opList = oldRecord.getOperationInformationList();
        opList.add(OperationInformation.submitOp(vo.getUserPhone(), vo.getMsg()));
        newRecord.setOperationInformationList(opList);

        //存入新纪录
        //原纪录标记为已归档
        investigationRecordDao.updateRecordState(oldRecord.getIdNumber(), InvestigationRecord.REVIEWED, InvestigationRecord.OUT_OF_DATE);
        investigationRecordDao.insertInvestigationRecord(newRecord);

        //TODO 将修改请求加入消息队列

    }

    @Override
    public boolean reviewRecord(ReviewVo reviewVo) {

        //如果待审核调查记录表不存在
        if (!investigationRecordDao.existInvestigationRecord(reviewVo.getIdNumber(), InvestigationRecord.UNDER_REVIEW)) {
            log.error("未审核记录不存在");
            throw new RecordException("未审核记录不存在");
        }
        //如果审核人不存在
        if(!userDao.existUser(reviewVo.getReviewerPhone())){
            log.error("审核人不存在");
            throw new UserException("审核人不存在");
        }

        InvestigationRecord record = investigationRecordDao.selectInvestigationRecord(reviewVo.getIdNumber(), InvestigationRecord.UNDER_REVIEW);

        //审核结果与版本信息
        record.setState(reviewVo.isPass() ? InvestigationRecord.REVIEWED : InvestigationRecord.NOT_PASS);
        record.setVersion(reviewVo.getReviewerPhone() + new Date().toString());

        //生成操作记录
        List<OperationInformation> opList = record.getOperationInformationList();
        opList.add(OperationInformation.reviewOp(reviewVo.getReviewerPhone(), reviewVo.getMsg(), reviewVo.isPass() ? "审核通过" : "审核不通过"));
        record.setOperationInformationList(opList);

        //更新数据库记录
        investigationRecordDao.reviewRecord(record);

        return true;
    }

    @Override
    public void export2Word(String idNumber, HttpServletResponse response) {
        InvestigationRecord record = investigationRecordDao.selectInvestigationRecord(idNumber, InvestigationRecord.REVIEWED);
        try {
            WordUtil.writeDocx(record, response);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }
}
