package com.example.survey.service.impl;

import com.example.survey.dao.InvestigationRecordDao;
import com.example.survey.dao.RoleDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.LoginDto;
import com.example.survey.dto.UserDto;
import com.example.survey.entity.Role;
import com.example.survey.entity.User;
import com.example.survey.enumeration.AuthEnum;
import com.example.survey.exception.UserException;
import com.example.survey.service.UserService;
import com.example.survey.util.TokenUtil;
import com.example.survey.vo.*;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author Pope
 */
@Log4j2
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserDao userDao;
    @Autowired
    private RoleDao roleDao;
    @Autowired
    private InvestigationRecordDao investigationRecordDao;


    @Override
    public LoginDto matchAuth(LoginVo loginVo) {
        User user = userDao.selectUser(loginVo.getPhoneNumber());
        if (user == null) {
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }
        if (!loginVo.getPassword().equals(user.getPassword())) {
            log.error("密码错误");
            throw new UserException("密码错误");
        }

        LoginDto loginDto = new LoginDto();
        //判断是否已经登录过
        if (TokenUtil.existKey(user.getPhoneNumber())) {
            String oldToken = (String) TokenUtil.get(user.getPhoneNumber());
            if (TokenUtil.existKey(oldToken)) {
                //已经登录，将旧token过期，
                TokenUtil.expireKey(oldToken);
            }
        }

        //生成新的token并存入redis
        String newToken = UUID.randomUUID().toString();
        TokenUtil.set(user.getPhoneNumber(), newToken);

        //生成角色列表
        Set<String> roleNameSet = new HashSet<>();
        for (Role role : user.getRoleList()) {
            Set<String> routeSet = new HashSet<>();
            for (AuthEnum authEnum : role.getAuthoritySet()) {
                routeSet.addAll(authEnum.getRoutes());
            }
            TokenUtil.setWithoutExpireTime(role.getName(), routeSet);
            roleNameSet.add(role.getName());
        }
        TokenUtil.set(newToken, roleNameSet);

        loginDto.setToken(newToken);
        loginDto.setUsername(user.getUsername());
        loginDto.setIdNumber(user.getIdNumber());
        loginDto.setPhoneNumber(user.getPhoneNumber());
        List<RoleVo> roleVoList = new ArrayList<>();
        for (Role role : user.getRoleList()) {
            roleVoList.add(new RoleVo(role));
        }
        loginDto.setRoleList(roleVoList);
        loginDto.setDepartmentList(user.getDepartmentList());
        loginDto.setAdministrativeArea(user.getAdministrativeArea());

        return loginDto;
    }

    @Override
    public boolean addUser(CreateUserVo createUserVo) {
        User user = new User(createUserVo);

        //初始化为流调人员权限
//        Role investigatorRole = roleDao.selectRole("流调人员");
//        List<Role> roleList = user.getRoleList();
//        roleList.add(investigatorRole);
//        user.setRoleList(roleList);
        return userDao.insertUser(user);
    }

    @Override
    public void modifyRole(UserRoleVo userRoleVo) {
        List<Role> roleList = new ArrayList<>();
        for (String roleName : userRoleVo.getRoleNameList()) {
            roleList.add(roleDao.selectRole(roleName));
        }

        userDao.updateUserRole(userRoleVo.getPhoneNumber(), roleList);
    }

    @Override
    public List<UserDto> listUserLimit(String username, String phoneNumber, int currentPage, int pageSize) {
        List<User> userList = userDao.listUserLimit(username, phoneNumber, currentPage * pageSize, pageSize);
        List<UserDto> userDtoList = new LinkedList<>();
        for (User user : userList) {
            userDtoList.add(new UserDto(user));
        }
        return userDtoList;
    }

    @Override
    public boolean deleteUser(String phoneNumber) {
        if (!userDao.existUser(phoneNumber)) {
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }
        userDao.deleteUser(phoneNumber);

        investigationRecordDao.updateManyRecordUserPhone(phoneNumber, null);

        if (TokenUtil.existKey(phoneNumber)) {
            String token = (String) TokenUtil.get(phoneNumber);
            TokenUtil.expireKey(phoneNumber);
            TokenUtil.expireKey(token);
        }
        return true;
    }

    @Override
    public void modifyUserInfo(UserInfoVo userInfoVo) {
        if (!userDao.existUser(userInfoVo.getPhoneNumber())) {
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }
        userDao.modifyUser(userInfoVo);
    }

    @Override
    public void resetPwd(String phoneNumber) {
        if (!userDao.existUser(phoneNumber)) {
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }
        userDao.resetPwd(phoneNumber);
    }


}
